package quotes.local;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;

import quotes.Channel;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */

public class GraphicalDisplay extends JPanel implements Observer {

	private static int latest_instance_number = 0;
	private int instance_number;

	/** Default colors, used if there is no provided custom color.*/
	private static Color[] colors = { Color.GREEN, Color.RED, Color.PINK, Color.BLUE };
	
	private List custom_colors = new LinkedList();
	
	/** The different types of rendering.*/
	public static final int LINES = 0, HISTOGRAM = 1;

	private BufferedSubscriber displayed_subscriber;

	private int type;
	private TinyConverter converter;

	/** Creates a new Graphical Display.*/
	public GraphicalDisplay() {
		super();
		instance_number = latest_instance_number++;
		converter = new TinyConverter();
	}

	/** Sets the rendering option.
	 * @param type the rendering option.
	 */
	public void setType(int type) {
		switch (type) {
			case LINES :
			case HISTOGRAM :
				this.type = type;
			default :
		}
	}
	
	/** Adds a custom color that will be used during the rendering process.
	 * @param color the custom color.
	 */
	public void addGraphColor(Color color) {
		custom_colors.add(color);
	}


	/**
	 * @see java.awt.Component#paintComponent(Graphics)
	 */
	public void paintComponent(Graphics graphics) {

		Rectangle bounds = getBounds();

		if (displayed_subscriber == null) { return; }
		if (bounds == null) { return; }

		converter.setViewDimensions(bounds.width, bounds.height);

		int bffr_sz = displayed_subscriber.getBufferSize();

		converter.setMax(bffr_sz - 1 , getMax());
		converter.setMin(0, getMin());

		graphics.setColor(Color.darkGray);
		graphics.fillRect(0, 0, bounds.width, bounds.height);

		graphics.setColor(Color.black);

		int zero_x = converter.convertX(0);
		int zero_y = converter.convertY(0);
		graphics.drawLine(0, zero_y, bounds.width, zero_y);
		graphics.drawLine(zero_x, 0, zero_x, bounds.height);

		int channel_number = -1;
		for (Enumeration enm = displayed_subscriber.getChannels();
			enm.hasMoreElements();
			) {
			channel_number++;
			
			if (channel_number < custom_colors.size()) {
				graphics.setColor((Color) custom_colors.get(channel_number));
			} else {
				graphics.setColor(colors[channel_number % colors.length]);
			}

			Channel chnnl = (Channel) enm.nextElement();
			int cursor = displayed_subscriber.getIndexOfLatestReceivedValueOnChannel(chnnl);

			graphics.drawString(
				chnnl.toString(),
				10,
				bounds.height - 5 - 12 * channel_number);

			int start = cursor;
			double previous_x = 0, previous_y;
			
			Object obj_value = displayed_subscriber.getBufferedValue(chnnl , ((start - 1) % bffr_sz));
			previous_y = (obj_value == null) ? 0 : ((Double) obj_value).doubleValue();

			for (int i = start, j = 0; j < bffr_sz ; i++, j++) {

				i = (i % bffr_sz);

				switch (type) {

					case LINES :
						Object obj_vl = displayed_subscriber.getBufferedValue(chnnl , i);
						double next_value = (obj_vl == null) ? 0 : ((Double) obj_vl).doubleValue();

						if ((next_value != 0.0d) && (previous_y != 0.0d)) {
							graphics.drawLine(
								converter.convertX(previous_x),
								converter.convertY(previous_y),
								converter.convertX(j),
								converter.convertY(next_value));
						}

						previous_x = j;
						previous_y = next_value;
						break;

					case HISTOGRAM :
						Object obj_vl_ = displayed_subscriber.getBufferedValue(chnnl , i);
						double y = (obj_vl_ == null) ? 0 : ((Double) obj_vl_).doubleValue();
						int y_height =
							converter.convertYDistance(Math.abs(y));
						if (y > 0) {
							graphics.fillRect(
								converter.convertX(j),
								converter.convertY(y),
								converter.convertXDistance(1) - 1,
								y_height);
						} else {
							graphics.fillRect(
								converter.convertX(j),
								converter.convertY(0),
								converter.convertXDistance(1) - 1,
								y_height);
						}
						break;

					default :

				}

			}
		}

		//Drawing reticuli
		graphics.setColor(Color.black);

		for (int i = 0; i < bffr_sz ; i++) {
			int x = converter.convertX(i);
			graphics.drawLine(x , bounds.height , x , bounds.height - 3 - ((i % 5 == 0) ? 2 : 0));
		}

		for (int i = -30; i < 30; i++) {
			int y = converter.convertY(i);
			graphics.drawLine(0, y, 2 + ((i % 5 == 0) ? 2 : 0), y);
			graphics.drawString("" + i, 3, y + 1);
		}

	}

	private double getMax() { return getExtremum(true); }
	private double getMin() { return getExtremum(false); }	

	private double getExtremum(boolean max) {
		double rslt = (max) ? Double.MIN_VALUE : Double.MAX_VALUE;
		int bffr_sz = displayed_subscriber.getBufferSize();
		for (Enumeration enm = displayed_subscriber.getChannels() ; enm.hasMoreElements() ;) {
			Channel chnnnl = (Channel) enm.nextElement();
			for (int i = 0 ; i < bffr_sz ; i++) {
				Double vl = (Double) displayed_subscriber.getBufferedValue(chnnnl , i);
				double value = (vl == null) ? 0 : vl.doubleValue();
				rslt = (max) ? Math.max(value , rslt) : Math.min(value , rslt);
			}
		}
		return rslt;
	}

	/**
	 * @see java.util.Observer#update(Observable, Object)
	 */
	public void update(Observable o, Object arg) {
		
		if (!(o instanceof BufferedSubscriber)) { return; }
		
		displayed_subscriber = (BufferedSubscriber) o;
		repaint();
	}

	/**
	 * @see java.awt.Component#getPreferredSize()
	 */
	public Dimension getPreferredSize() {
		return new Dimension(200, 200);
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "[" + instance_number + "] " + super.toString();
	}

} // GraphicalDisplay

class TinyConverter {

	private double min_x, min_y, max_x, max_y;
	private int width, height;

	void setViewDimensions(int width, int height) {
		this.width = width;
		this.height = height;
	}

	void setMax(double max_x, double max_y) {
		this.max_x = max_x;
		this.max_y = max_y;
	}

	void setMin(double min_x, double min_y) {
		this.min_x = min_x;
		this.min_y = min_y;
	}

	int convertX(double x) {
		double zoom = Math.abs(max_x - min_x) / width;
		return (int) ((min_x + x) / zoom);
	}

	int convertY(double y) {
		double zoom = Math.abs(max_y - min_y) / height;
		return (int) ((max_y - y) / zoom);
	}

	int convertXDistance(double distance) {
		double zoom = Math.abs(max_x - min_x) / width;
		return (int) (distance / zoom);
	}

	int convertYDistance(double distance) {
		double zoom = Math.abs(max_y - min_y) / height;
		return (int) (distance / zoom);
	}

}
