package test;

import java.util.Iterator;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.naming.Context;

import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.repository.ResourceNotFoundException;
import org.objectweb.kilim.tools.jmx.ComponentMBean;
import mx4j.adaptor.rmi.jrmp.JRMPAdaptorMBean;
import mx4j.util.StandardMBeanProxy;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.helpers.KilimHelper;

import com.sun.jdmk.comm.HtmlAdaptorServer;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class LauncherJMX {

   /** Launches the application.
    * @param args nothing is needed here.
    */
   public static void main(String[] args) {
      
      if (args.length < 1) {
	 System.out.println("Syntax : java LauncherJMX template_name");
	 System.exit(-1);
      }


      MBeanServer server = launchJMXServer();
      Component compo = createMainComponent(args[0]);
      registerComponent(server , compo);
		
      launchRMIAdaptor(server);
      launchSunHTTPAdaptor(server);
   }
	
   /**
    * Method createMainComponent.
    * @param name :
    * @return Component
    */
   public static Component createMainComponent(String name) {

      Component compo = null;
      Object res = null;

      try {
			
	 compo = KilimHelper.newComponent(name , LauncherJMX.class);

      } catch (KilimException e) {
	 e.printStackTrace();
	 System.exit(-1);
      } catch (ResourceNotFoundException e) {
	 e.printStackTrace();
	 System.exit(-1);
      }
		
      return compo;
   }

   /**
    * Method launchJMXServer.
    * @return MBeanServer
    */
   public static MBeanServer launchJMXServer() {

      // CREATE the MBeanServer
      //
      System.out.println("\n\tCREATE the MBeanServer.");
      MBeanServer server = MBeanServerFactory.createMBeanServer();
      ComponentMBean.setDefaultMBeanServer(server);
      return server;
   }
	
   private static void launchRMIAdaptor(MBeanServer server) {
		
      System.out.println("\n\tCREATE, REGISTER and START a new RMI adaptor:");	
	
      try {
  	
	 // Create and start the naming service
      	 ObjectName naming = new ObjectName("Naming:type=rmiregistry");
       	 server.createMBean("mx4j.tools.naming.NamingService", naming, null);
       	 server.invoke(naming, "start", null, null);
	
	 // Create the JRMP adaptor
	 ObjectName adaptor = new ObjectName("Adaptor:protocol=JRMP");
	 System.out.println("\tOBJECT NAME           = " + adaptor);
	 server.createMBean("mx4j.adaptor.rmi.jrmp.JRMPAdaptor", adaptor, null);
	 JRMPAdaptorMBean mbean = (JRMPAdaptorMBean) StandardMBeanProxy.create(JRMPAdaptorMBean.class, server, adaptor);
	
	 // Set the JNDI name with which will be registered
	 String jndiName = "jrmp";
	 mbean.setJNDIName(jndiName);
		  
	 // Optionally, you can specify the JNDI properties, 
	 // instead of having in the classpath a jndi.properties file
	 mbean.putJNDIProperty(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.rmi.registry.RegistryContextFactory");
	 mbean.putJNDIProperty(Context.PROVIDER_URL, "rmi://localhost:1099");
		  

	 // Registers the JRMP adaptor in JNDI and starts it
	 mbean.start();
	  
      } catch (Exception e) {
	 e.printStackTrace();
	 System.exit(-1);
      }
   }
	
	
   private static void launchSunHTTPAdaptor(MBeanServer server) {
      // CREATE and START a new HTML adaptor
      System.out.println("\n\tCREATE, REGISTER and START a new HTML adaptor:");
			
      HtmlAdaptorServer html = new HtmlAdaptorServer();
		
      ObjectName html_name = null;
      try {
	 html_name = new ObjectName("Adaptor:name=html,port=8080");
	 System.out.println("\tOBJECT NAME           = " + html_name);
	 server.registerMBean(html, html_name);
      } catch (Exception e) {
	 System.out.println("\t!!! Could not create the HTML adaptor !!!");
	 e.printStackTrace();
	 System.exit(-1);
      }

      html.start();
   }
	
   /**
    * Method registerComponentAndSubComponents.
    * @param server :
    * @param compo :
    */
   public static void registerComponentAndSubComponents(MBeanServer server , Component compo) {
      registerComponent(server , compo);
      for (Iterator it = compo.getSubComponents()	; it.hasNext() ;) {	
	 registerComponentAndSubComponents(server , (Component) it.next());
      }
   }
	
   /**
    * Method registerComponent.
    * @param server :
    * @param compo :
    */
   public static void registerComponent(MBeanServer server , Component compo) {
      ComponentMBean compo_mbean = new ComponentMBean(compo);
		
      ObjectName component_name = null;
		
      try {
	 component_name = new ObjectName("Component:name=ROOT");
	 System.out.println("\tOBJECT NAME           = " + component_name);
	 server.registerMBean(compo_mbean, component_name);
      } catch (Exception e) {
	 System.out.println("\t!!! Could not create the HTML adaptor !!!");
	 e.printStackTrace();
	 System.exit(-1);
      }
		
   }
	

}
